@@ -15,7 +15,11 @@ module Puppet
       using based on the platform you are on, but you can override it
       using the `provider` parameter; each provider defines what it
       requires in order to function, and you must meet those requirements
-      to use a given provider."
+      to use a given provider.
+      
+      **Autorequires:** If Puppet is managing the files specified as a package's
+      `adminfile`, `responsefile`, or `source`, the package resource will autorequire
+      those files."
 
     feature :installable, "The provider can install packages.",
       :methods => [:install]
@@ -109,8 +113,6 @@ module Puppet
       # Override the parent method, because we've got all kinds of
       # funky definitions of 'in sync'.
       def insync?(is)
-        @should ||= []
-
         @latest ||= nil
         @lateststamp ||= (Time.now.to_i - 1000)
         # Iterate across all of the should values, and see how they
